#requires -version 2.0

#TODO: Add /Domain functionality

Function Get-NetUser {

<#
.Synopsis
Get user info from the NET USER command.
.Description
This function takes the output from the legacy NET USER command and creates
PowerShell pipelined output.

.Parameter Name
The name of the user account. If you don't specify a name then you'll get a
list of users.
.Parameter Computername
The name of the computer to query. The computer must be configured for
PowerShell remoting. 

.Example
PS C:\> get-netuser Administrator

Get information about the local Administrator account
.Example
PS C:\> get-netuser | Select -ExpandProperty Users

Get a list of of all users from the local system.


.Example
PS C:\> get-netuser | select -ExpandProperty users | get-netuser -verbose

Get details for all users from the local system.
#>

[cmdletbinding()]

Param(
[Parameter(Position=0,ValueFromPipeline=$True)]
[string]$Name,
[string]$computername
)

Process {
    #if no $name then get all users
    if ($name) {
    Write-Verbose "Creating scriptblock for $name"
    $sb={Param([string]$user)
        Write-Verbose ("Getting net local user {0}" -f $user)
        #filter out blanks
        #redirect errors
        $data=net user $user 2>"$env:temp\netuser.err" | 
        Where {$_ -AND $_ -notmatch "command completed"}
        
        if ((Get-Item "$env:temp\netuser.err").Length -gt 0) {
            $myerror=get-content "$env:temp\netuser.err" | out-string
            Write-Error $myerror
        }

        #define a hash table to hold values
        $hash=@{Computername=$env:computername}
        #split each line into properties
        #replace any spaces in property names with nothing
        $data | Foreach { 
          $parts=$_ -split ("\s{2,}")
          $prop=$parts[0].Replace(" ","")
          $value=$parts[1]
          $hash.Add($prop,$value) }

        #write the object
        if ($data) {
            New-Object -type PSObject -property $hash
        }
     } #sb
    }
    else {
    Write-Verbose "Creating all users scriptblock"
    $sb={
        Write-Verbose "Getting all users"
        #filter out the dashed line and the command completed message
        #redirect errors
        $data=Net User 2>"$env:temp\netuser.err"| where {$_ -AND $_ -notmatch "---|command completed"}
        
        if ((Get-Item "$env:temp\netuser.err").Length -gt 0) {
            $myerror=get-content "$env:temp\netuser.err" | out-string
            Write-Error $myerror
        }
        
        write-verbose ($data | out-string)
        #match on any nonspace characters after \\
        Write-Verbose ("Looking for computername in {0}" -f $data[0])
        [regex]$r="\\\\\S+"
        $MatchComputer=$r.match($data[0])
        
        if ($MatchComputer.Success) {
            $computer=$MatchComputer.value.Substring(2)
        }
        else {
           #when using remoting there is no computername so use the environmental variable
            $computer=$env:computername
        }
        
        Write-Verbose "Found $computer"
        
        #split each line on the space. $users will be an array
        $users=$data | Select -skip 1 | foreach { $_ -split "\s" | where {$_}}

        #create a new object
        Write-Verbose "Creating the object"
        new-object -type psobject -prop @{Computername=$computer;Users=$users}
        } #sb
    } #else
    
    #run the scriptblock
    if ($computername -AND $computername -ne $env:computername) {
        Write-Verbose "Running a remote command on $computername"
        invoke-command $sb -ComputerName $computername -HideComputerName -ArgumentList $name | select * -exclude RunspaceID
    }
    else {
        Write-Verbose "Running a local command"
        invoke-command $sb -ArgumentList $name
    }
    
} #process

} #end function

